chiark / gitweb /
[PATCH] Fix gentoo bug #76056 (fb device group permissions.)
[elogind.git] / namedev.c
index 7045637e79e5f3423d6a4bd005dfd0b23ee513b9..332e2d57d80f9a773ef0579ce2eb8f0c723c0dad 100644 (file)
--- a/namedev.c
+++ b/namedev.c
 
 static struct sysfs_attribute *find_sysfs_attribute(struct sysfs_class_device *class_dev, struct sysfs_device *sysfs_device, char *attr);
 
-LIST_HEAD(config_device_list);
-LIST_HEAD(perm_device_list);
-
-
 /* compare string with pattern (supports * ? [0-9] [!A-Z]) */
 static int strcmp_pattern(const char *p, const char *s)
 {
@@ -100,18 +96,6 @@ static int strcmp_pattern(const char *p, const char *s)
        return 1;
 }
 
-static struct perm_device *find_perm_entry(const char *name)
-{
-       struct perm_device *perm;
-
-       list_for_each_entry(perm, &perm_device_list, node) {
-               if (strcmp_pattern(perm->name, name))
-                       continue;
-               return perm;
-       }
-       return NULL;
-}
-
 /* extract possible {attr} and move str behind it */
 static char *get_format_attribute(char **str)
 {
@@ -217,6 +201,12 @@ static void apply_format(struct udevice *udev, char *string, size_t maxsize,
 
 
                switch (c) {
+               case 'p':
+                       if (strlen(udev->devpath) == 0)
+                               break;
+                       strfieldcatmax(string, udev->devpath, maxsize);
+                       dbg("substitute kernel name '%s'", udev->kernel_name);
+                       break;
                case 'b':
                        if (strlen(udev->bus_id) == 0)
                                break;
@@ -306,6 +296,15 @@ static void apply_format(struct udevice *udev, char *string, size_t maxsize,
                                strfieldcatmax(string, temp2, maxsize);
                        }
                        break;
+               case 'N':
+                       if (udev->tmp_node[0] == '\0') {
+                               dbg("create temporary device node for callout");
+                               snprintf(udev->tmp_node, NAME_SIZE-1, "%s/.tmp-%u-%u", udev_root, udev->major, udev->minor);
+                               udev_make_node(udev, udev->tmp_node, udev->major, udev->minor, 0600, 0, 0);
+                       }
+                       strfieldcatmax(string, udev->tmp_node, maxsize);
+                       dbg("substitute temporary device node name '%s'", udev->tmp_node);
+                       break;
                default:
                        dbg("unknown substitution type '%%%c'", c);
                        break;
@@ -665,8 +664,7 @@ static int match_rule(struct udevice *udev, struct config_device *dev,
 
                /* check for matching result of external program */
                if (dev->result[0] != '\0') {
-                       dbg("check for " FIELD_RESULT
-                           " dev->result='%s', udev->program_result='%s'",
+                       dbg("check for " FIELD_RESULT " dev->result='%s', udev->program_result='%s'",
                            dev->result, udev->program_result);
                        if (strcmp_pattern(dev->result, udev->program_result) != 0) {
                                dbg(FIELD_RESULT " is not matching");
@@ -687,7 +685,6 @@ try_parent:
                dbg("sysfs_device->path='%s'", sysfs_device->path);
                dbg("sysfs_device->bus_id='%s'", sysfs_device->bus_id);
        }
-
 }
 
 int namedev_name_device(struct udevice *udev, struct sysfs_class_device *class_dev)
@@ -695,10 +692,8 @@ int namedev_name_device(struct udevice *udev, struct sysfs_class_device *class_d
        struct sysfs_class_device *class_dev_parent;
        struct sysfs_device *sysfs_device = NULL;
        struct config_device *dev;
-       struct perm_device *perm;
        char *pos;
 
-       udev->mode = 0;
        dbg("class_dev->name='%s'", class_dev->name);
 
        /* Figure out where the "device"-symlink is at.  For char devices this will
@@ -735,18 +730,36 @@ int namedev_name_device(struct udevice *udev, struct sysfs_class_device *class_d
        list_for_each_entry(dev, &config_device_list, node) {
                dbg("process rule");
                if (match_rule(udev, dev, class_dev, sysfs_device) == 0) {
-                       /* empty name and symlink will not create any node */
-                       if (dev->name[0] == '\0' && dev->symlink[0] == '\0') {
-                               info("configured rule in '%s' at line %i applied, '%s' is ignored",
+
+                       /* empty name, symlink and perms will not create any node */
+                       if (dev->name[0] == '\0' && dev->symlink[0] == '\0' &&
+                           dev->mode == 0000 && dev->owner[0] == '\0' && dev->group[0] == '\0') {
+                               info("configured rule in '%s[%i]' applied, '%s' is ignored",
                                     dev->config_file, dev->config_line, udev->kernel_name);
                                return -1;
                        }
 
-                       /* collect symlinks for the final matching rule */
+                       /* apply permissions */
+                       if (dev->mode != 0000) {
+                               udev->mode = dev->mode;
+                               dbg("applied mode=%#o to '%s'", udev->mode, udev->kernel_name);
+                       }
+                       if (dev->owner[0] != '\0') {
+                               strfieldcpy(udev->owner, dev->owner);
+                               apply_format(udev, udev->owner, sizeof(udev->owner), class_dev, sysfs_device);
+                               dbg("applied owner='%s' to '%s'", udev->owner, udev->kernel_name);
+                       }
+                       if (dev->group[0] != '\0') {
+                               strfieldcpy(udev->group, dev->group);
+                               apply_format(udev, udev->group, sizeof(udev->group), class_dev, sysfs_device);
+                               dbg("applied group='%s' to '%s'", udev->group, udev->kernel_name);
+                       }
+
+                       /* collect symlinks for this or the final matching rule */
                        if (dev->symlink[0] != '\0') {
                                char temp[NAME_SIZE];
 
-                               info("configured rule in '%s' at line %i applied, added symlink '%s'",
+                               info("configured rule in '%s[%i]' applied, added symlink '%s'",
                                     dev->config_file, dev->config_line, dev->symlink);
                                strfieldcpy(temp, dev->symlink);
                                apply_format(udev, temp, sizeof(temp), class_dev, sysfs_device);
@@ -762,7 +775,7 @@ int namedev_name_device(struct udevice *udev, struct sysfs_class_device *class_d
                                    (udev->type != 'b' || udev->kernel_number[0] != '\0'))
                                        continue;
 
-                               info("configured rule in '%s' at line %i applied, '%s' becomes '%s'",
+                               info("configured rule in '%s[%i]' applied, '%s' becomes '%s'",
                                     dev->config_file, dev->config_line, udev->kernel_name, dev->name);
 
                                strfieldcpy(udev->name, dev->name);
@@ -775,61 +788,25 @@ int namedev_name_device(struct udevice *udev, struct sysfs_class_device *class_d
                                        goto exit;
 
                                udev->partitions = dev->partitions;
-                               udev->mode = dev->mode;
-                               strfieldcpy(udev->owner, dev->owner);
-                               strfieldcpy(udev->group, dev->group);
 
-                               goto perms;
+                               dbg("name, '%s' is going to have owner='%s', group='%s', mode=%#o partitions=%i",
+                                   udev->name, udev->owner, udev->group, udev->mode, udev->partitions);
+
+                               goto exit;
                        }
                }
        }
 
        /* no rule matched, so we use the kernel name */
        strfieldcpy(udev->name, udev->kernel_name);
+       dbg("no rule found, use kernel name '%s'", udev->name);
 
-       if (udev->type == 'n')
-               goto exit;
-
-perms:
-       /* apply permissions from permissions file to empty fields */
-       perm = find_perm_entry(udev->name);
-       if (perm != NULL) {
-               if (udev->mode == 0000)
-                       udev->mode = perm->mode;
-               if (udev->owner[0] == '\0')
-                       strfieldcpy(udev->owner, perm->owner);
-               if (udev->group[0] == '\0')
-                       strfieldcpy(udev->group, perm->group);
+exit:
+       if (udev->tmp_node[0] != '\0') {
+               dbg("removing temporary device node");
+               unlink_secure(udev->tmp_node);
+               udev->tmp_node[0] = '\0';
        }
 
-       /* apply permissions from config to empty fields */
-       if (udev->mode == 0000)
-               udev->mode = default_mode;
-       if (udev->owner[0] == '\0')
-               strfieldcpy(udev->owner, default_owner);
-       if (udev->group[0] == '\0')
-               strfieldcpy(udev->group, default_group);
-
-       dbg("name, '%s' is going to have owner='%s', group='%s', mode = %#o",
-           udev->name, udev->owner, udev->group, udev->mode);
-
-exit:
        return 0;
 }
-
-int namedev_init(void)
-{
-       int retval;
-
-       retval = namedev_init_rules();
-       if (retval)
-               return retval;
-
-       retval = namedev_init_permissions();
-       if (retval)
-               return retval;
-
-       dump_config_dev_list();
-       dump_perm_dev_list();
-       return retval;
-}